# Nonaticipativity constraint formulation of stochastic problem 
# for portfolio selection
param T;

set SCENARIOS;

set TIMESTEPS ordered :=1..T;
set ENDSTEPS ordered := TIMESTEPS diff {first(TIMESTEPS)};
set ASSETS;

set BINDINGS within TIMESTEPS cross SCENARIOS cross SCENARIOS;

param asset_developpment{ENDSTEPS,ASSETS,SCENARIOS};
param initial_assets{ASSETS};
param inflow{ENDSTEPS,SCENARIOS};
param prob{SCENARIOS};

var owned{TIMESTEPS,ASSETS,SCENARIOS} >=0;
var buy{ENDSTEPS,ASSETS,SCENARIOS} >=0;
var sell{ENDSTEPS,ASSETS,SCENARIOS} >=0;

maximize end_wealth:
	 sum{ a in ASSETS, s in SCENARIOS} prob[s]*owned[last(TIMESTEPS),a];

subject to mass_conservation{t in ENDSTEPS, s in SCENARIOS}:
     sum{ a in ASSETS}(buy[t,a,s]-sell[t,a,s])=inflow[t,s];

subject to developpment{s in SCENARIOS,
	                t in TIMESTEPS, 
                        a in ASSETS
                           :t != first(TIMESTEPS)}:
     owned[t,a,s]=
     	  asset_developpment[t,a,s]*owned[prev(t),a,s]
     	  +buy[t,a]-sell[t,a,s];

subject to initial_owned{a in ASSETS, s in SCENARIOS}:
     owned[first(TIMESTEPS),a,s]=initial_assets[a];

subject to nonant_1{(t, s1,s2) in BINDINGS}:
	owned[t,s1]=owned[t,s2]; 

subject to nonant_2{(t, s1,s2) in BINDINGS}:
	sold[t,s1]=sold[t,s2]; 

subject to nonant_3{(t, s1,s2) in BINDINGS}:
	bought[t,s1]=bought[t,s2]; 

